@@ -127,6 +127,10 @@ class ShadowBeamlineRenderer(AbstractBeamlineRenderer):
                         if oe_end.IDUMMY == 0: alpha = int(oe_end.ALPHA)  # oe not changed by shadow, angles in deg changed to rad
                         else:                  alpha = int(oe_end.ALPHA * TODEG)
 
+                        if alpha < 0: alpha = int(360 + alpha)
+
+                        if not alpha in [0, 90, 180, 270]: raise ValueError("Rendering not possible, orientation angle not supported")
+
                         if previous_orientation == Orientations.UP:
                             if alpha == 0:     orientation = Orientations.UP
                             elif alpha == 90:  orientation = Orientations.LEFT
@@ -197,8 +201,12 @@ class ShadowBeamlineRenderer(AbstractBeamlineRenderer):
                                                    label=None, aspect_ratio_modifier=aspect_ratio_modifier)
                             else:
                                 if oe_end.F_REFRAC == 0:
-                                    if oe_end.IDUMMY == 0: inclination = (90 - oe_end.T_INCIDENCE) / TODEG # oe not changed by shadow, angles in deg changed to rad
-                                    else:                  inclination  = (numpy.pi/2) - oe_end.T_INCIDENCE
+                                    if oe_end.IDUMMY == 0:
+                                        inclination_in = (90 - oe_end.T_INCIDENCE) / TODEG # oe not changed by shadow, angles in deg changed to rad
+                                        inclination_out = (90 - oe_end.T_REFLECTION) / TODEG
+                                    else:
+                                        inclination_in  = (numpy.pi/2) - oe_end.T_INCIDENCE
+                                        inclination_out  = (numpy.pi/2) - oe_end.T_REFLECTION
 
                                     if oe_end.FHIT_C == 1:
                                         width = oe_start.RWIDX1 + oe_start.RWIDX2
@@ -217,7 +225,7 @@ class ShadowBeamlineRenderer(AbstractBeamlineRenderer):
                                         color = OpticalElementsColors.MIRROR
                                         label = "Mirror"
 
-                                    absolute_inclination, beam_horizontal_inclination, beam_vertical_inclination = get_inclinations(orientation, inclination, beam_vertical_inclination, beam_horizontal_inclination)
+                                    absolute_inclination, beam_horizontal_inclination, beam_vertical_inclination = get_inclinations(orientation, inclination_in, inclination_out, beam_vertical_inclination, beam_horizontal_inclination)
 
                                     self.add_optical_element(centers, limits, oe_index=oe_index,
                                                              distance=oe_total_distance, height=height, shift=shift,
@@ -275,7 +283,7 @@ class ShadowBeamlineRenderer(AbstractBeamlineRenderer):
                                            label="Compound OE (" + oe_type + ")", aspect_ratio_modifier=aspect_ratio_modifier)
                         else:
                             if oe_type == 'CRLs/Transfocator':
-                                length    = source_segment + image_segment - (oe_start.list[0].T_SOURCE - oe_start.list[-1].T_IMAGE)
+                                length    = source_segment + image_segment - (oe_start.list[0].T_SOURCE + oe_start.list[-1].T_IMAGE)
                                 color     = OpticalElementsColors.LENS
                             else:
                                 length    =  100 / self.workspace_units_to_mm
